@@ -163,7 +163,7 @@ void AtticaProvider::listOfCategoriesLoaded(Attica::BaseJob *listJob)
         emit providerInitialized(this);
         emit categoriesMetadataLoded(categoryMetadataList);
     } else {
-        emit signalError(i18n("All categories are missing"));
+        emit signalErrorCode(KNSCore::ConfigFileError, i18n("All categories are missing"), QVariant());
     }
 }
 
@@ -273,7 +273,11 @@ void AtticaProvider::categoryContentsLoaded(BaseJob *job)
     EntryInternal::List entries;
     TagsFilterChecker checker(tagFilter());
     TagsFilterChecker downloadschecker(downloadTagFilter());
-    Q_FOREACH (const Content &content, contents) {
+    for (const Content &content : contents) {
+        if (!content.isValid()) {
+            qCDebug(KNEWSTUFFCORE) << "Filtered out an invalid entry. This suggests something is not right on the originating server. Please contact the administrators of" << name() << "and inform them there is an issue with content in the category or categories" << mCurrentRequest.categories;
+            continue;
+        }
         if (checker.filterAccepts(content.tags())) {
             bool filterAcceptsDownloads = true;
             if (content.downloads() > 0) {
@@ -437,13 +441,13 @@ bool AtticaProvider::jobSuccess(Attica::BaseJob *job) const
     qCDebug(KNEWSTUFFCORE) << "job error: " << job->metadata().error() << " status code: " << job->metadata().statusCode() << job->metadata().message();
 
     if (job->metadata().error() == Attica::Metadata::NetworkError) {
-        emit signalError(i18n("Network error %1: %2", job->metadata().statusCode(), job->metadata().statusString()));
+        emit signalErrorCode(KNSCore::NetworkError, i18n("Network error %1: %2", job->metadata().statusCode(), job->metadata().statusString()), job->metadata().statusCode());
     }
     if (job->metadata().error() == Attica::Metadata::OcsError) {
         if (job->metadata().statusCode() == 200) {
-            emit signalError(i18n("Too many requests to server. Please try again in a few minutes."));
+            emit signalErrorCode(KNSCore::OcsError, i18n("Too many requests to server. Please try again in a few minutes."), job->metadata().statusCode());
         } else {
-            emit signalError(i18n("Unknown Open Collaboration Service API error. (%1)", job->metadata().statusCode()));
+            emit signalErrorCode(KNSCore::OcsError, i18n("Unknown Open Collaboration Service API error. (%1)", job->metadata().statusCode()), job->metadata().statusCode());
         }
     }
     return false;
