@@ -17,34 +17,34 @@
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "installation_p.h"
+#include "installation.h"
 
 #include <QDir>
 #include <QFile>
 #include <QTemporaryFile>
-#include <QtCore/QProcess>
+#include <QProcess>
 #include <QUrlQuery>
+#include <QDesktopServices>
 
 #include "qmimedatabase.h"
 #include "karchive.h"
 #include "kzip.h"
 #include "ktar.h"
-#include "kio/job.h"
 #include "krandom.h"
 #include "kshell.h"
-#include "kmessagebox.h" // TODO get rid of message box
-#include <KRun>
+
 #include <qstandardpaths.h>
 #include "klocalizedstring.h"
-#include <knewstuff_debug.h>
+#include <knewstuffcore_debug.h>
 
-#include "core/security_p.h"
+#include "jobs/filecopyjob.h"
+#include "question.h"
 #ifdef Q_OS_WIN
 #include <windows.h>
 #include <shlobj.h>
 #endif
 
-using namespace KNS3;
+using namespace KNSCore;
 
 Installation::Installation(QObject *parent)
     : QObject(parent)
@@ -62,11 +62,14 @@ bool Installation::readConfig(const KConfigGroup &group)
     // FIXME: read out only when actually installing as a performance improvement?
     QString uncompresssetting = group.readEntry("Uncompress", QStringLiteral("never"));
     // support old value of true as equivalent of always
-    if (uncompresssetting == QLatin1String("true")) {
+    if (uncompresssetting == QLatin1String("subdir")) {
+        uncompresssetting = QStringLiteral("subdir");
+    }
+    else if (uncompresssetting == QLatin1String("true")) {
         uncompresssetting = QStringLiteral("always");
     }
-    if (uncompresssetting != QLatin1String("always") && uncompresssetting != QLatin1String("archive") && uncompresssetting != QLatin1String("never")) {
-        qCritical() << "invalid Uncompress setting chosen, must be one of: always, archive, or never" << endl;
+    if (uncompresssetting != QLatin1String("always") && uncompresssetting != QLatin1String("archive") && uncompresssetting != QLatin1String("never") && uncompresssetting != QLatin1String("subdir")) {
+        qCCritical(KNEWSTUFFCORE) << "invalid Uncompress setting chosen, must be one of: subdir, always, archive, or never" << endl;
         return false;
     }
     uncompression = uncompresssetting;
@@ -81,6 +84,13 @@ bool Installation::readConfig(const KConfigGroup &group)
         xdgTargetDirectory = QStringLiteral("wallpapers");
     }
 
+    // also, ensure wallpapers are decompressed into subdirectories
+    // this ensures that wallpapers with multiple resolutions continue to function
+    // as expected
+    if (xdgTargetDirectory == QStringLiteral("wallpapers")) {
+        uncompression = QStringLiteral("subdir");
+    }
+
     installPath = group.readEntry("InstallPath", QString());
     absoluteInstallPath = group.readEntry("AbsoluteInstallPath", QString());
     customName = group.readEntry("CustomName", false);
@@ -91,7 +101,7 @@ bool Installation::readConfig(const KConfigGroup &group)
             xdgTargetDirectory.isEmpty() &&
             installPath.isEmpty() &&
             absoluteInstallPath.isEmpty()) {
-        qCritical() << "No installation target set";
+        qCCritical(KNEWSTUFFCORE) << "No installation target set";
         return false;
     }
 
@@ -104,7 +114,7 @@ bool Installation::readConfig(const KConfigGroup &group)
         } else if (checksumpolicy == QLatin1String("always")) {
             checksumPolicy = Installation::CheckAlways;
         } else {
-            qCritical() << "The checksum policy '" + checksumpolicy + "' is unknown." << endl;
+            qCCritical(KNEWSTUFFCORE) << QStringLiteral("The checksum policy '") + checksumpolicy + QStringLiteral("' is unknown.") << endl;
             return false;
         }
     }
@@ -118,7 +128,7 @@ bool Installation::readConfig(const KConfigGroup &group)
         } else if (signaturepolicy == QLatin1String("always")) {
             signaturePolicy = Installation::CheckAlways;
         } else {
-            qCritical() << "The signature policy '" + signaturepolicy + "' is unknown." << endl;
+            qCCritical(KNEWSTUFFCORE) << QStringLiteral("The signature policy '") + signaturepolicy + QStringLiteral("' is unknown.") << endl;
             return false;
         }
     }
@@ -130,13 +140,13 @@ bool Installation::readConfig(const KConfigGroup &group)
         } else if (scopeString == QLatin1String("system")) {
             scope = ScopeSystem;
         } else {
-            qCritical() << "The scope '" + scopeString + "' is unknown." << endl;
+            qCCritical(KNEWSTUFFCORE) << QStringLiteral("The scope '") + scopeString + QStringLiteral("' is unknown.") << endl;
             return false;
         }
 
         if (scope == ScopeSystem) {
             if (!installPath.isEmpty()) {
-                qCritical() << "System installation cannot be mixed with InstallPath." << endl;
+                qCCritical(KNEWSTUFFCORE) << "System installation cannot be mixed with InstallPath." << endl;
                 return false;
             }
         }
@@ -164,12 +174,12 @@ bool Installation::isRemote() const
     return true;
 }
 
-void Installation::install(EntryInternal entry)
+void Installation::install(const EntryInternal& entry)
 {
     downloadPayload(entry);
 }
 
-void Installation::downloadPayload(const KNS3::EntryInternal &entry)
+void Installation::downloadPayload(const KNSCore::EntryInternal &entry)
 {
     if (!entry.isValid()) {
         emit signalInstallationFailed(i18n("Invalid item."));
@@ -178,7 +188,7 @@ void Installation::downloadPayload(const KNS3::EntryInternal &entry)
     QUrl source = QUrl(entry.payload());
 
     if (!source.isValid()) {
-        qCritical() << "The entry doesn't have a payload." << endl;
+        qCCritical(KNEWSTUFFCORE) << "The entry doesn't have a payload." << endl;
         emit signalInstallationFailed(i18n("Download of item failed: no download URL for \"%1\".", entry.name()));
         return;
     }
@@ -186,7 +196,7 @@ void Installation::downloadPayload(const KNS3::EntryInternal &entry)
     // FIXME no clue what this is supposed to do
     if (isRemote()) {
         // Remote resource
-        qCDebug(KNEWSTUFF) << "Relaying remote payload '" << source << "'";
+        qCDebug(KNEWSTUFFCORE) << "Relaying remote payload '" << source << "'";
         install(entry, source.toDisplayString(QUrl::PreferLocalFile));
         emit signalPayloadLoaded(source);
         // FIXME: we still need registration for eventual deletion
@@ -194,15 +204,15 @@ void Installation::downloadPayload(const KNS3::EntryInternal &entry)
     }
 
     QString fileName(source.fileName());
-    QTemporaryFile tempFile(QDir::tempPath() + "/XXXXXX-" + fileName);
+    QTemporaryFile tempFile(QDir::tempPath() + QStringLiteral("/XXXXXX-") + fileName);
     if (!tempFile.open()) {
         return;    // ERROR
     }
     QUrl destination = QUrl::fromLocalFile(tempFile.fileName());
-    qCDebug(KNEWSTUFF) << "Downloading payload" << source << "to" << destination;
+    qCDebug(KNEWSTUFFCORE) << "Downloading payload" << source << "to" << destination;
 
     // FIXME: check for validity
-    KIO::FileCopyJob *job = KIO::file_copy(source, destination, -1, KIO::Overwrite | KIO::HideProgressInfo);
+    FileCopyJob *job = FileCopyJob::file_copy(source, destination, -1, JobFlag::Overwrite | JobFlag::HideProgressInfo);
     connect(job,
             &KJob::result,
             this, &Installation::slotPayloadResult);
@@ -220,77 +230,78 @@ void Installation::slotPayloadResult(KJob *job)
         if (job->error()) {
             emit signalInstallationFailed(i18n("Download of \"%1\" failed, error: %2", entry.name(), job->errorString()));
         } else {
-            KIO::FileCopyJob *fcjob = static_cast<KIO::FileCopyJob *>(job);
+            FileCopyJob *fcjob = static_cast<FileCopyJob *>(job);
 
             // check if the app likes html files - disabled by default as too many bad links have been submitted to opendesktop.org
             if (!acceptHtml) {
                 QMimeDatabase db;
                 QMimeType mimeType = db.mimeTypeForFile(fcjob->destUrl().toLocalFile());
                 if (mimeType.inherits(QStringLiteral("text/html")) || mimeType.inherits(QStringLiteral("application/x-php"))) {
-                    if (KMessageBox::questionYesNo(0, i18n("The downloaded file is a html file. This indicates a link to a website instead of the actual download. Would you like to open the site with a browser instead?"), i18n("Possibly bad download link"))
-                            == KMessageBox::Yes) {
-                        KRun::runUrl(fcjob->srcUrl(), QStringLiteral("text/html"), Q_NULLPTR);
+                    Question question;
+                    question.setQuestion(i18n("The downloaded file is a html file. This indicates a link to a website instead of the actual download. Would you like to open the site with a browser instead?"));
+                    question.setTitle(i18n("Possibly bad download link"));
+                    if(question.ask() == Question::YesResponse) {
+                        QDesktopServices::openUrl(fcjob->srcUrl());
                         emit signalInstallationFailed(i18n("Downloaded file was a HTML file. Opened in browser."));
-                        entry.setStatus(Entry::Invalid);
+                        entry.setStatus(KNS3::Entry::Invalid);
                         emit signalEntryChanged(entry);
                         return;
                     }
                 }
             }
 
-            install(entry, fcjob->destUrl().toLocalFile());
             emit signalPayloadLoaded(fcjob->destUrl());
+            install(entry, fcjob->destUrl().toLocalFile());
         }
     }
 }
 
-void Installation::install(KNS3::EntryInternal entry, const QString &downloadedFile)
+void KNSCore::Installation::install(KNSCore::EntryInternal entry, const QString& downloadedFile)
 {
-    qCDebug(KNEWSTUFF) << "Install: " << entry.name() << " from " << downloadedFile;
+    qCDebug(KNEWSTUFFCORE) << "Install: " << entry.name() << " from " << downloadedFile;
 
     if (entry.payload().isEmpty()) {
-        qCDebug(KNEWSTUFF) << "No payload associated with: " << entry.name();
+        qCDebug(KNEWSTUFFCORE) << "No payload associated with: " << entry.name();
         return;
     }
 
-    // FIXME: first of all, do the security stuff here
     // this means check sum comparison and signature verification
     // signature verification might take a long time - make async?!
     /*
     if (checksumPolicy() != Installation::CheckNever) {
         if (entry.checksum().isEmpty()) {
             if (checksumPolicy() == Installation::CheckIfPossible) {
-                qCDebug(KNEWSTUFF) << "Skip checksum verification";
+                qCDebug(KNEWSTUFFCORE) << "Skip checksum verification";
             } else {
-                qCritical() << "Checksum verification not possible" << endl;
+                qCCritical(KNEWSTUFFCORE) << "Checksum verification not possible" << endl;
                 return false;
             }
         } else {
-            qCDebug(KNEWSTUFF) << "Verify checksum...";
+            qCDebug(KNEWSTUFFCORE) << "Verify checksum...";
         }
     }
     if (signaturePolicy() != Installation::CheckNever) {
         if (entry.signature().isEmpty()) {
             if (signaturePolicy() == Installation::CheckIfPossible) {
-                qCDebug(KNEWSTUFF) << "Skip signature verification";
+                qCDebug(KNEWSTUFFCORE) << "Skip signature verification";
             } else {
-                qCritical() << "Signature verification not possible" << endl;
+                qCCritical(KNEWSTUFFCORE) << "Signature verification not possible" << endl;
                 return false;
             }
         } else {
-            qCDebug(KNEWSTUFF) << "Verify signature...";
+            qCDebug(KNEWSTUFFCORE) << "Verify signature...";
         }
     }
     */
 
-    QString targetPath = targetInstallationPath(downloadedFile);
+    QString targetPath = targetInstallationPath();
     QStringList installedFiles = installDownloadedFileAndUncompress(entry, downloadedFile, targetPath);
 
     if (installedFiles.isEmpty()) {
-        if (entry.status() == Entry::Installing) {
-            entry.setStatus(Entry::Downloadable);
-        } else if (entry.status() == Entry::Updating) {
-            entry.setStatus(Entry::Updateable);
+        if (entry.status() == KNS3::Entry::Installing) {
+            entry.setStatus(KNS3::Entry::Downloadable);
+        } else if (entry.status() == KNS3::Entry::Updating) {
+            entry.setStatus(KNS3::Entry::Updateable);
         }
         emit signalEntryChanged(entry);
         emit signalInstallationFailed(i18n("Could not install \"%1\": file not found.", entry.name()));
@@ -299,44 +310,32 @@ void Installation::install(KNS3::EntryInternal entry, const QString &downloadedF
 
     entry.setInstalledFiles(installedFiles);
 
+    auto installationFinished = [this, entry]() {
+        EntryInternal newentry = entry;
+        // update version and release date to the new ones
+        if (newentry.status() == KNS3::Entry::Updating) {
+            if (!newentry.updateVersion().isEmpty()) {
+                newentry.setVersion(newentry.updateVersion());
+            }
+            if (newentry.updateReleaseDate().isValid()) {
+                newentry.setReleaseDate(newentry.updateReleaseDate());
+            }
+        }
+
+        newentry.setStatus(KNS3::Entry::Installed);
+        emit signalEntryChanged(newentry);
+        emit signalInstallationFinished();
+    };
     if (!postInstallationCommand.isEmpty()) {
-        if (installedFiles.size() == 1) {
-            runPostInstallationCommand(installedFiles.first());
-        } else {
-            runPostInstallationCommand(targetPath);
-        }
+        QProcess* p = runPostInstallationCommand(installedFiles.size() == 1 ? installedFiles.first() : targetPath);
+        connect(p, static_cast<void(QProcess::*)(int)>(&QProcess::finished), this, installationFinished);
+    } else {
+        installationFinished();
     }
-
-    // ==== FIXME: security code below must go above, when async handling is complete ====
-
-    // FIXME: security object lifecycle - it is a singleton!
-    Security *sec = Security::ref();
-
-    connect(sec,
-            &Security::validityResult,
-            this, &Installation::slotInstallationVerification);
-
-    // FIXME: change to accept filename + signature
-    sec->checkValidity(QString());
-
-    // update version and release date to the new ones
-    if (entry.status() == Entry::Updating) {
-        if (!entry.updateVersion().isEmpty()) {
-            entry.setVersion(entry.updateVersion());
-        }
-        if (entry.updateReleaseDate().isValid()) {
-            entry.setReleaseDate(entry.updateReleaseDate());
-        }
-    }
-
-    entry.setStatus(Entry::Installed);
-    emit signalEntryChanged(entry);
-    emit signalInstallationFinished();
 }
 
-QString Installation::targetInstallationPath(const QString &payloadfile)
+QString Installation::targetInstallationPath() const
 {
-    QString installpath(payloadfile);
     QString installdir;
 
     if (!isRemote()) {
@@ -369,11 +368,11 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
             if (scope == ScopeUser) {
                 installdir = QStandardPaths::writableLocation(location);
             } else { // system scope
-                installdir = QStandardPaths::standardLocations(location).last();
+                installdir = QStandardPaths::standardLocations(location).constLast();
             }
             pathcounter++;
         }
-        if (!targetDirectory.isEmpty() && targetDirectory != "/") {
+        if (!targetDirectory.isEmpty() && targetDirectory != QLatin1String("/")) {
             if (scope == ScopeUser) {
                 installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + targetDirectory + QLatin1Char('/');
             } else { // system scope
@@ -381,7 +380,7 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
             }
             pathcounter++;
         }
-        if (!xdgTargetDirectory.isEmpty() && xdgTargetDirectory != "/") {
+        if (!xdgTargetDirectory.isEmpty() && xdgTargetDirectory != QLatin1String("/")) {
             installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + xdgTargetDirectory + QLatin1Char('/');
             pathcounter++;
         }
@@ -389,12 +388,12 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
 #if defined(Q_OS_WIN)
             WCHAR wPath[MAX_PATH + 1];
             if (SHGetFolderPathW(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, wPath) == S_OK) {
-                installdir = QString::fromUtf16((const ushort *) wPath) + QLatin1Char('/') + installpath + QLatin1Char('/');
+                installdir = QString::fromUtf16((const ushort *) wPath) + QLatin1Char('/') + installPath + QLatin1Char('/');
             } else {
-                installdir =  QDir::home().path() + QLatin1Char('/') + installPath + QLatin1Char('/');
+                installdir =  QDir::homePath() + QLatin1Char('/') + installPath + QLatin1Char('/');
             }
 #else
-            installdir = QDir::home().path() + QLatin1Char('/') + installPath + QLatin1Char('/');
+            installdir = QDir::homePath() + QLatin1Char('/') + installPath + QLatin1Char('/');
 #endif
             pathcounter++;
         }
@@ -404,11 +403,11 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
         }
 
         if (pathcounter != 1) {
-            qCritical() << "Wrong number of installation directories given." << endl;
+            qCCritical(KNEWSTUFFCORE) << "Wrong number of installation directories given." << endl;
             return QString();
         }
 
-        qCDebug(KNEWSTUFF) << "installdir: " << installdir;
+        qCDebug(KNEWSTUFFCORE) << "installdir: " << installdir;
 
         // create the dir if it doesn't exist (QStandardPaths doesn't create it, unlike KStandardDirs!)
         QDir().mkpath(installdir);
@@ -417,9 +416,8 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
     return installdir;
 }
 
-QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryInternal  &entry, const QString &payloadfile, const QString installdir)
+QStringList Installation::installDownloadedFileAndUncompress(const KNSCore::EntryInternal  &entry, const QString &payloadfile, const QString installdir)
 {
-    QString installpath(payloadfile);
     // Collect all files that were installed
     QStringList installedFiles;
 
@@ -427,12 +425,11 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
         bool isarchive = true;
 
         // respect the uncompress flag in the knsrc
-        if (uncompression == QLatin1String("always") || uncompression == QLatin1String("archive")) {
+        if (uncompression == QLatin1String("always") || uncompression == QLatin1String("archive") || uncompression == QLatin1String("subdir")) {
             // this is weird but a decompression is not a single name, so take the path instead
-            installpath = installdir;
             QMimeDatabase db;
             QMimeType mimeType = db.mimeTypeForFile(payloadfile);
-            qCDebug(KNEWSTUFF) << "Postinstallation: uncompress the file";
+            qCDebug(KNEWSTUFFCORE) << "Postinstallation: uncompress the file";
 
             // FIXME: check for overwriting, malicious archive entries (../foo) etc.
             // FIXME: KArchive should provide "safe mode" for this!
@@ -449,7 +446,7 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
                        || mimeType.inherits(QStringLiteral("application/x-compressed-tar"))) {
                 archive.reset(new KTar(payloadfile));
             } else {
-                qCritical() << "Could not determine type of archive file '" << payloadfile << "'";
+                qCCritical(KNEWSTUFFCORE) << "Could not determine type of archive file '" << payloadfile << "'";
                 if (uncompression == QLatin1String("always")) {
                     return QStringList();
                 }
@@ -459,7 +456,7 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
             if (isarchive) {
                 bool success = archive->open(QIODevice::ReadOnly);
                 if (!success) {
-                    qCritical() << "Cannot open archive file '" << payloadfile << "'";
+                    qCCritical(KNEWSTUFFCORE) << "Cannot open archive file '" << payloadfile << "'";
                     if (uncompression == QLatin1String("always")) {
                         return QStringList();
                     }
@@ -469,15 +466,20 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
 
                 if (isarchive) {
                     const KArchiveDirectory *dir = archive->directory();
-                    //if there is more than an item in the file,
+                    //if there is more than an item in the file, and we are requested to do so
                     //put contents in a subdirectory with the same name as the file
-                    if (dir->entries().count() > 1) {
+                    QString installpath;
+                    if (uncompression == QLatin1String("subdir") && dir->entries().count() > 1) {
                         installpath = installdir + QLatin1Char('/') + QFileInfo(archive->fileName()).baseName();
+                    } else {
+                        installpath = installdir;
                     }
-                    dir->copyTo(installpath);
 
-                    installedFiles << archiveEntries(installpath, dir);
-                    installedFiles << installpath + QLatin1Char('/');
+                    if (dir->copyTo(installpath)) {
+                        installedFiles << archiveEntries(installpath, dir);
+                        installedFiles << installpath + QLatin1Char('/');
+                    } else
+                        qCWarning(KNEWSTUFFCORE) << "could not install" << entry.name() << "to" << installpath;
 
                     archive->close();
                     QFile::remove(payloadfile);
@@ -485,22 +487,24 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
             }
         }
 
-        qCDebug(KNEWSTUFF) << "isarchive: " << isarchive;
+        qCDebug(KNEWSTUFFCORE) << "isarchive: " << isarchive;
 
-        if (uncompression == QLatin1String("never") || (uncompression == QLatin1String("archive") && !isarchive)) {
+        //some wallpapers are compressed, some aren't
+        if ((!isarchive && standardResourceDirectory == QLatin1String("wallpaper")) ||
+            (uncompression == QLatin1String("never") || (uncompression == QLatin1String("archive") && !isarchive))) {
             // no decompress but move to target
 
             /// @todo when using KIO::get the http header can be accessed and it contains a real file name.
             // FIXME: make naming convention configurable through *.knsrc? e.g. for kde-look.org image names
             QUrl source = QUrl(entry.payload());
-            qCDebug(KNEWSTUFF) << "installing non-archive from " << source.url();
+            qCDebug(KNEWSTUFFCORE) << "installing non-archive from " << source.url();
             QString installfile;
-            QString ext = source.fileName().section('.', -1);
+            QString ext = source.fileName().section(QLatin1Char('.'), -1);
             if (customName) {
                 installfile = entry.name();
-                installfile += '-' + entry.version();
+                installfile += QLatin1Char('-') + entry.version();
                 if (!ext.isEmpty()) {
-                    installfile += '.' + ext;
+                    installfile += QLatin1Char('.') + ext;
                 }
             } else {
                 // TODO HACK This is a hack, the correct way of fixing it would be doing the KIO::get
@@ -508,7 +512,7 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
                 // Randa this is not going to happen anytime soon (if ever) so go with the hack
                 if (source.url().startsWith(QLatin1String("http://newstuff.kde.org/cgi-bin/hotstuff-access?file="))) {
                     installfile = QUrlQuery(source).queryItemValue(QStringLiteral("file"));
-                    int lastSlash = installfile.lastIndexOf('/');
+                    int lastSlash = installfile.lastIndexOf(QLatin1Char('/'));
                     if (lastSlash >= 0) {
                         installfile = installfile.mid(lastSlash);
                     }
@@ -517,9 +521,9 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
                     installfile = source.fileName();
                 }
             }
-            installpath = installdir + QLatin1Char('/') + installfile;
+            QString installpath = installdir + QLatin1Char('/') + installfile;
 
-            qCDebug(KNEWSTUFF) << "Install to file " << installpath;
+            qCDebug(KNEWSTUFFCORE) << "Install to file " << installpath;
             // FIXME: copy goes here (including overwrite checking)
             // FIXME: what must be done now is to update the cache *again*
             //        in order to set the new payload filename (on root tag only)
@@ -527,22 +531,29 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
             // FIXME: for updates, we might need to force an overwrite (that is, deleting before)
             QFile file(payloadfile);
             bool success = true;
-            const bool update = ((entry.status() == Entry::Updateable) || (entry.status() == Entry::Updating));
+            const bool update = ((entry.status() == KNS3::Entry::Updateable) || (entry.status() == KNS3::Entry::Updating));
 
-            if (QFile::exists(installpath)) {
+            if (QFile::exists(installpath) && QDir::tempPath() != installdir) {
                 if (!update) {
-                    if (KMessageBox::warningContinueCancel(0, i18n("Overwrite existing file?") + "\n'" + installpath + '\'', i18n("Download File")) == KMessageBox::Cancel) {
+                    Question question(Question::ContinueCancelQuestion);
+                    question.setQuestion(i18n("Overwrite existing file?") + QStringLiteral("\n'") + installpath + QLatin1Char('\''));
+                    question.setTitle(i18n("Download File"));
+                    if(question.ask() != Question::ContinueResponse) {
                         return QStringList();
                     }
                 }
                 success = QFile::remove(installpath);
             }
             if (success) {
+                //remove in case it's already present and in a temporary directory, so we get to actually use the path again
+                if (installpath.startsWith(QDir::tempPath())) {
+                    file.remove(installpath);
+                }
                 success = file.rename(installpath);
-                qCDebug(KNEWSTUFF) << "move: " << file.fileName() << " to " << installpath;
+                qCDebug(KNEWSTUFFCORE) << "move: " << file.fileName() << " to " << installpath;
             }
             if (!success) {
-                qCritical() << "Cannot move file '" << payloadfile << "' to destination '"  << installpath << "'";
+                qCCritical(KNEWSTUFFCORE) << "Cannot move file '" << payloadfile << "' to destination '"  << installpath << "'";
                 return QStringList();
             }
             installedFiles << installpath;
@@ -551,24 +562,32 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
     return installedFiles;
 }
 
-void Installation::runPostInstallationCommand(const QString &installPath)
+QProcess* Installation::runPostInstallationCommand(const QString &installPath)
 {
     QString command(postInstallationCommand);
     QString fileArg(KShell::quoteArg(installPath));
     command.replace(QLatin1String("%f"), fileArg);
 
-    qCDebug(KNEWSTUFF) << "Run command: " << command;
+    qCDebug(KNEWSTUFFCORE) << "Run command: " << command;
 
-    int exitcode = QProcess::execute(command);
+    QProcess* ret = new QProcess(this);
+    connect(ret, static_cast<void(QProcess::*)(int)>(&QProcess::finished), this, [this, command](int exitcode){
+        if (exitcode) {
+            qCCritical(KNEWSTUFFCORE) << "Command '" << command << "' failed with code" << exitcode;
+        }
+        sender()->deleteLater();
+    });
 
-    if (exitcode) {
-        qCritical() << "Command failed" << endl;
-    }
+    QStringList args = KShell::splitArgs(command);
+    ret->setProgram(args.takeFirst());
+    ret->setArguments(args);
+    ret->start();
+    return ret;
 }
 
 void Installation::uninstall(EntryInternal entry)
 {
-    entry.setStatus(Entry::Deleted);
+    entry.setStatus(KNS3::Entry::Deleted);
 
     if (!uninstallCommand.isEmpty()) {
         foreach (const QString &file, entry.installedFiles()) {
@@ -581,22 +600,29 @@ void Installation::uninstall(EntryInternal entry)
                 int exitcode = QProcess::execute(command);
 
                 if (exitcode) {
-                    qCritical() << "Command failed" << endl;
+                    qCCritical(KNEWSTUFFCORE) << "Command failed" << command;
                 } else {
-                    qCDebug(KNEWSTUFF) << "Command executed successfully";
+                    qCDebug(KNEWSTUFFCORE) << "Command executed successfully: " << command;
                 }
             }
         }
     }
 
     foreach (const QString &file, entry.installedFiles()) {
-        if (file.endsWith('/')) {
+        if (file.endsWith(QLatin1Char('/'))) {
             QDir dir;
             bool worked = dir.rmdir(file);
             if (!worked) {
                 // Maybe directory contains user created files, ignore it
                 continue;
             }
+        } else if (file.endsWith(QLatin1String("/*"))) {
+            QDir dir(file.left(file.size()-2));
+            bool worked = dir.removeRecursively();
+            if (!worked) {
+                qCWarning(KNEWSTUFFCORE) << "Couldn't remove" << dir.path();
+                continue;
+            }
         } else {
             QFileInfo info(file);
             if (info.exists() || info.isSymLink()) {
@@ -618,31 +644,21 @@ void Installation::uninstall(EntryInternal entry)
 
 void Installation::slotInstallationVerification(int result)
 {
-    qCDebug(KNEWSTUFF) << "SECURITY result " << result;
-
-    //FIXME do something here ??? and get the right entry again
-    EntryInternal entry;
-
-    if (result & Security::SIGNED_OK) {
-        emit signalEntryChanged(entry);
-    } else {
-        emit signalEntryChanged(entry);
-    }
+    Q_UNUSED(result)
+    // Deprecated, was wired up to defunct Security class.
 }
 
 QStringList Installation::archiveEntries(const QString &path, const KArchiveDirectory *dir)
 {
     QStringList files;
     foreach (const QString &entry, dir->entries()) {
-        QString childPath = path + QLatin1Char('/') + entry;
-        if (dir->entry(entry)->isFile()) {
-            files << childPath;
-        }
+        const auto currentEntry = dir->entry(entry);
 
-        if (dir->entry(entry)->isDirectory()) {
-            const KArchiveDirectory *childDir = static_cast<const KArchiveDirectory *>(dir->entry(entry));
-            files << archiveEntries(childPath, childDir);
-            files << childPath + QLatin1Char('/');
+        const QString childPath = path + QLatin1Char('/') + entry;
+        if (currentEntry->isFile()) {
+            files << childPath;
+        } else if (currentEntry->isDirectory()) {
+            files << childPath + QStringLiteral("/*");
         }
     }
     return files;
